#include <OpenMS/FORMAT/MzMLFile.h>
#include <OpenMS/KERNEL/ConsensusMap.h>
#include <OpenMS/KERNEL/FeatureMap.h>
#include <OpenMS/FORMAT/ConsensusXMLFile.h>
#include <OpenMS/FORMAT/FeatureXMLFile.h>
#include <OpenMS/CHEMISTRY/AASequence.h>
#include <OpenMS/FORMAT/TextFile.h>

using namespace OpenMS;

/*
   finds the consensusFeature with the required peptide sequence and charge and returns TRUE
   returns FALSE if not found
*/
bool findConsensus(const ConsensusMap& cm, const String& pep_seq, const Int& charge, ConsensusFeature& cf)
{
  for (Size i=0; i<cm.size(); ++i)
  {
    // look at PeptideIdentifications:
    std::vector<PeptideIdentification> pep_ids = cm [ i ].getPeptideIdentifications();
    for (Size i_ids = 0; i_ids < pep_ids.size(); ++ i_ids)
    {
      if ((pep_ids[i_ids].getHits()[0].getSequence() == pep_seq) && cm[i].getCharge() == charge)
      {
        cf = cm[i];
        return true;
      }
    }
  }

  return false;
}

int main(int argc, char *argv[])
{

	if (argc!=4) 
	{
		std::cout << "Evaluate SILAC ratio data by comparing CSV file to simulation output\nUsage " << argv[0] << " <in_SIM_ConsensusMap.consensusXML> <in_CSV_from_PEPXML.result> <out_CSV_ratios>\n";
		return 0;
	}
	
  /*
  input CSV is expected to look like (tab separated):
  > seq charge observed_ratio


  output CSV is (tab separated) - one line for each SILAC pair:
  > expected_ratio observed_ratio light_SILAC_intensity 
  
  */

	ConsensusXMLFile f;
	ConsensusMap cm;
  f.load(String(argv[1]), cm);
	
	TextFile tf;
  tf.load(String(argv[2]));
  

  TextFile out_tf;
  out_tf.push_back("simRatio,1_simRatio,obsRatio,1_obsRatio,intensity,hasOverlappingFeature");

  Size count_wrongCFsize = 0;
  Size count_seqNotFound = 0;
  Size count_overlapping = 0;
  // search 
  for (Size i=0; i< tf.size(); ++i)
	{
    // split CSV line:
    StringList columns;
    tf[i].split("\t", columns);

    if (columns.size()!=3) throw Exception::InvalidSize(__FILE__, __LINE__, __PRETTY_FUNCTION__, columns.size());

    // parse CSV line
    String pep_seq=columns[0];
    Int pep_charge = columns[1].toInt();
    DoubleReal obs_ratio =  columns[2].toDouble();

    // find real SILAC ratio
    ConsensusFeature cf;
    
    if (! findConsensus(cm, pep_seq, pep_charge, cf))
    {
      ++count_seqNotFound;
      continue;
    }

    ConsensusFeature::HandleSetType fhs = cf.getFeatures ();

    if (fhs.size()!=2)
    {
      //throw Exception::InvalidSize(__FILE__, __LINE__, __PRETTY_FUNCTION__, fhs.size());
      ++count_wrongCFsize;
    }

    Peak2D f1 = *(fhs.begin());
    Peak2D f2 = *(++fhs.begin());

    // let f1 be the light feature
    if (f1.getMZ() > f2.getMZ())
    {
      Peak2D tmp = f1;
      f1 = f2;
      f2 = tmp;
    }
    // get theoretical ratio
    DoubleReal real_ratio = f1.getIntensity() / f2.getIntensity();
    
    // write result for SILAC pair
    // --> expected_ratio observed_ratio light_SILAC_intensity 
    DataValue has_overlapping_feature = cf.getMetaValue("has_overlapping_feature");
    if(has_overlapping_feature == DataValue::EMPTY || (Int) has_overlapping_feature == 0)
    {
      out_tf.push_back(String(real_ratio) + "," + String(1/real_ratio) + "," + String(obs_ratio) + "," + String(1/ obs_ratio) + "," + String(f1.getIntensity()) + ",0" );
    }
    else
    {
      out_tf.push_back(String(real_ratio) + "," + String(1/real_ratio) + "," + String(obs_ratio) + "," + String(1/ obs_ratio) + "," + String(f1.getIntensity()) + ",1" );
      ++count_overlapping;
    }
  }

  out_tf.store(String(argv[3]));

  // stats:
  std::cout << "pairs found: " << out_tf.size() << "\n";
  std::cout << "  overlapping: " << count_overlapping << "\n";
  std::cout << "invalid total: " << (count_wrongCFsize + count_seqNotFound) << " / " << (out_tf.size()*2) << " (" << (float(count_wrongCFsize + count_seqNotFound)/float(out_tf.size()*2)*100) << "%)\n";
	std::cout << "  invalid CF size: " << count_wrongCFsize << "\n";
  std::cout << "  invalid seq (not found): " << count_seqNotFound << "\n";

	return 0;
}
